%computes slant angle correction analytically through the relatioship of
%radial distance and refraction. 
close all
clc
clear
%Phantom 4 Pro RTK Specs
f = 8.8; %focal lengh in mm 
formatA = 13.186; %long side of format, 13.186 mm
formatB = 8.79; %short side, 8.79 mm
sensdiag = 15.85; %sensor diagonal 15.85 mm
FOV = 84; %degrees
r_max = tand(FOV/2)*f;
%r_max = sensdiag/2;
r = (0:0.1:r_max);
r_scale = r/r_max;
%refraction coefficients
n1= 1.0003; %air
%seawater coefficients
a = -1.50156*10^-6;
b = 1.07085*10^-7;
c = -4.27594 *10^-5;
d = -1.60476*10^-4;
e = 1.39807;
T = 28; %water temp in C
lambda = 500; %blue light = 450 to 495 nm red light 620 to 750 nm
n2 = a*T^2+b*lambda^2 + c*T + d*lambda + e;
%n2 = 1.34;
lambda_j = (450:10:750);
n_j = a*T^2+b.*lambda_j.^2 + c*T + d.*lambda_j + e;

plot(lambda_j,n_j)
title('Wavelength and Refraction Index')
xlabel('Wavelength (nm)')
%refraction angle
theta_i = atan(r/f);
theta_ik = asin((n1/n2).*sin(theta_i));

% figure
% plot(r,theta_i)
% hold on
% plot(r,theta_ik)
% title('Incidence-Refraction Angles and Radial Distance')
% xlabel('Radial Distance from Principal Point, r (mm)')
% ylabel('Angle (rad)')
% legend('Incidence Angle','Refraction Angle')

Dd_ratio = cos(theta_ik);
dD_ratio = 1./cos(theta_ik);
p = polyfit(r,Dd_ratio,1);
py = polyval(p,r);
p_s = polyfit(r_scale,Dd_ratio,1);
py_s = polyval(p_s,r_scale);
Mdl1 = LinearModel.fit(r_scale,Dd_ratio);
Rsq = Mdl1.Rsquared.Ordinary;

pd = polyfit(r,dD_ratio,1);
pyd = polyval(pd,r);
p_sd = polyfit(r_scale,dD_ratio,1);
py_sd = polyval(p_sd,r_scale);
Mdl1d = LinearModel.fit(r_scale,dD_ratio);
Rsqd = Mdl1d.Rsquared.Ordinary;

% tilt evaluation
% theta_a = rad2deg(theta_i);
% theta_c = 84-theta_a;
theta_t = 0:.01:3;
M = length(theta_t);
DdA = zeros(M,1);
DdC = zeros(M,1);
DdB = zeros(M,1);
i = 1;
tilt_pos = zeros(length(theta_i),M+1); %struct to hold incidence angle and D_A and D_C per tilt value. 
tilt_neg = zeros(length(theta_i),M+1); %struct to hold incidence angle and D_A and D_C per tilt value.

for j = 1:length(theta_i)
    theta_a = rad2deg(theta_i(j));
    theta_c = theta_a;
    while i<M+1
    DdA(i) = cosd(asind((n1/n2)*sind(theta_a - theta_t(i))));
    DdC(i) = cosd(asind((n1/n2)*sind(theta_a + theta_t(i))));
    DdB(i) = cosd(theta_t(i));
    tilt_pos(j,1) = theta_a;
    tilt_neg(j,1) = theta_c;
    tilt_pos(j,i+1) = DdA(i);
    tilt_neg(j,i+1) = DdC(i);
    i=i+1;
    end
    i = 1;
end

pos = polyfit(r_scale,tilt_pos(:,M),1);
pyp = polyval(pos,r);
p_sp = polyfit(r_scale,tilt_pos(:,M),1);
py_sp = polyval(p_sp,r_scale);
Mdl1p = LinearModel.fit(r_scale,tilt_pos(:,M));
Rsqp = Mdl1p.Rsquared.Ordinary;

neg = polyfit(r_scale,tilt_neg(:,M),1);
pyn = polyval(neg,r);
p_sn = polyfit(r_scale,tilt_neg(:,M),1);
py_sn = polyval(p_sn,r_scale);
Mdl1n = LinearModel.fit(r_scale,tilt_neg(:,M));
Rsqn = Mdl1n.Rsquared.Ordinary;
% figure 
% hold on 
% for k = 2:M
% plot(r_scale,tilt_pos(:,k),'+');
% plot(r_scale,tilt_neg(:,k),'-.')
% end
% plot(r_scale,Dd_ratio)
figure %2
error_pos = tilt_pos(:,M)-Dd_ratio';
error_neg = tilt_neg(:,M)-Dd_ratio';
errorbar(r_scale,Dd_ratio,error_neg,error_pos)
xlabel('Radial Distance Ratio, \rho')
ylabel('Corrected/Uncorrected Depth Ratio, \delta')
title(sprintf('Depth Correction Ratio - With Tilt %0.2f degrees',theta_t(M)))

figure %3
hold on
plot(r_scale,tilt_pos(:,M),'+')
plot(r_scale,tilt_neg(:,M),'-.')
legend('Positive Tilt','Negative Tilt')
%plot(r_scale,py_sp)
%plot(r_scale,py_sn)
xlabel('Radial Distance Ratio, \rho ')
ylabel('Corrected/Uncorrected Depth Ratio, \delta')
title(sprintf('Depth Correction Ratio - With Tilt %0.2f degrees',theta_t(M)))
annotation('textbox',[0.1 .3 .1 .1],'Position',[0.15 .3 .1 .1],'String',sprintf('Pos: \\delta = %f \\rho + %f \n R^2 = %0.4f \n  Neg: \\delta = %f \\rho + %f \n R^2 = %0.4f',p_sp(1),p_sp(2),Rsqp,p_sn(1),p_sn(2),Rsqn))


% figure
% hold on
% plot(r,dD_ratio)
% xlabel('Radial Distance from Principal Point, r (mm)')
% ylabel('Uncorrected/Corrected Depth Ratio d/D')
% plot(r,py,'r--')
% legend('D/d','linear fit')
% annotation('textbox',[0.1 .3 .1 .1],'Position',[0.15 .7 .1 .1],'String',sprintf('D/d = %fR %f',p(1),p(2)));
% title(sprintf('Radial Distance and Depth Ratio for %dnm light',lambda))

figure %4
hold on
plot(r_scale,Dd_ratio)
xlabel('Radial Distance Ratio, \rho')
ylabel('Corrected/Uncorrected Depth Ratio, \delta')
plot(r_scale,py_s,'r--')
legend('\delta','linear fit')
annotation('textbox',[0.1 .3 .1 .1],'Position',[0.5 .7 .1 .1],'String',sprintf('\\delta = %f \\rho + %f \n R^2 = %0.4f',p_s(1),p_s(2),Rsq));   

% figure %5
% hold on
% plot(r_scale,dD_ratio)
% xlabel('Radial Distance Ratio, r_r (unitless)')
% ylabel('Uncorrected/Corrected Depth Ratio d/D')
% plot(r_scale,py_sd,'r-')
% legend('d/D','linear fit')
% annotation('textbox',[0.1 .3 .1 .1],'Position',[0.15 .7 .1 .1],'String',sprintf('d/D = %f r_r + %f \n R^2 = %0.4f',p_sd(1),p_sd(2),Rsqd));   
% %title('Radial Distance Ratio and Refracted Depth Correction')

%simplified trig relationship:
% figure
% AA = n2^2*(r_scale.^2+f^2);
% AB = n1^2.*r_scale.^2;
% Dd_trig = sqrt((AA-AB)./AA);
% plot(r_scale,Dd_trig)